package sun.java2d.opengl;

import sun.java2d.loops.GraphicsPrimitive;
import sun.java2d.pipe.ParallelogramPipe;
import sun.java2d.pipe.SpanIterator;
import java.awt.geom.Path2D;
import sun.java2d.pipe.BufferedContext;
import sun.java2d.InvalidPipeException;
import sun.java2d.SunGraphics2D;
import sun.java2d.pipe.RenderQueue;
import sun.java2d.pipe.BufferedRenderPipe;

class OGLRenderer extends BufferedRenderPipe
{
    OGLRenderer(RenderQueue rq) {
        super(rq);
    }
    
    @Override
    protected void validateContext(SunGraphics2D sg2d) {
        int flags = sg2d.paint.getTransparency() == 1;
        OGLSurfaceData oglSurfaceData;
        try {
            oglSurfaceData = (OGLSurfaceData)sg2d.surfaceData;
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sg2d.surfaceData);
        }
        BufferedContext.validateContext(oglSurfaceData, oglSurfaceData, sg2d.getCompClip(), sg2d.composite, null, sg2d.paint, sg2d, flags);
    }
    
    @Override
    protected void validateContextAA(SunGraphics2D sg2d) {
        int flags = 0;
        OGLSurfaceData oglSurfaceData;
        try {
            oglSurfaceData = (OGLSurfaceData)sg2d.surfaceData;
        }
        catch (ClassCastException ex) {
            throw new InvalidPipeException("wrong surface data type: " + sg2d.surfaceData);
        }
        BufferedContext.validateContext(oglSurfaceData, oglSurfaceData, sg2d.getCompClip(), sg2d.composite, null, sg2d.paint, sg2d, flags);
    }
    
    void copyArea(SunGraphics2D sunGraphics2D, int x, int x2, int x3, int x4, int x5, int x6) {
        this.rq.lock();
        try {
            int flags = sunGraphics2D.surfaceData.getTransparency() == 1;
            OGLSurfaceData oglSurfaceData;
            try {
                oglSurfaceData = (OGLSurfaceData)sunGraphics2D.surfaceData;
            }
            catch (ClassCastException ex) {
                throw new InvalidPipeException("wrong surface data type: " + sunGraphics2D.surfaceData);
            }
            BufferedContext.validateContext(oglSurfaceData, oglSurfaceData, sunGraphics2D.getCompClip(), sunGraphics2D.composite, null, null, null, flags);
            this.rq.ensureCapacity(28);
            this.buf.putInt(30);
            this.buf.putInt(x).putInt(x2).putInt(x3).putInt(x4);
            this.buf.putInt(x5).putInt(x6);
        }
        finally {
            this.rq.unlock();
        }
    }
    
    @Override
    protected native void drawPoly(int[] p0, int[] p1, int p2, boolean p3, int p4, int p5);
    
    OGLRenderer traceWrap() {
        return new OGLRenderer.Tracer(this);
    }
    
    static void access$100(OGLRenderer oglRenderer, SunGraphics2D sg2d, int[] xPoints, int[] yPoints, int nPoints, boolean isClosed) {
        oglRenderer.drawPoly(sg2d, xPoints, yPoints, nPoints, isClosed);
    }
    
    static void access$200(OGLRenderer oglRenderer, SunGraphics2D sg2d, Path2D.Float p2df, int transx, int transy) {
        oglRenderer.drawPath(sg2d, p2df, transx, transy);
    }
    
    static void access$300(OGLRenderer oglRenderer, SunGraphics2D sg2d, Path2D.Float p2df, int transx, int transy) {
        oglRenderer.fillPath(sg2d, p2df, transx, transy);
    }
    
    static void access$400(OGLRenderer oglRenderer, SunGraphics2D sg2d, SpanIterator si, int transx, int transy) {
        oglRenderer.fillSpans(sg2d, si, transx, transy);
    }
    
    private class Tracer extends OGLRenderer
    {
        private OGLRenderer oglr;
        
        Tracer(OGLRenderer oglr) {
            super(oglr.rq);
            this.oglr = oglr;
        }
        
        @Override
        public ParallelogramPipe getAAParallelogramPipe() {
            return new ParallelogramPipe() {
                
                @Override
                public void fillParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10) {
                    GraphicsPrimitive.tracePrimitive("OGLFillAAParallelogram");
                    realpipe.fillParallelogram(sunGraphics2D, n, n2, n3, n4, n5, n6, n7, n8, n9, n10);
                }
                
                @Override
                public void drawParallelogram(SunGraphics2D sunGraphics2D, double n, double n2, double n3, double n4, double n5, double n6, double n7, double n8, double n9, double n10, double n11, double n12) {
                    GraphicsPrimitive.tracePrimitive("OGLDrawAAParallelogram");
                    realpipe.drawParallelogram(sunGraphics2D, n, n2, n3, n4, n5, n6, n7, n8, n9, n10, n11, n12);
                }
            };
        }
        
        @Override
        protected void validateContext(SunGraphics2D sunGraphics2D) {
            this.oglr.validateContext(sunGraphics2D);
        }
        
        @Override
        public void drawLine(SunGraphics2D sg2d, int x1, int y1, int x2, int y2) {
            GraphicsPrimitive.tracePrimitive("OGLDrawLine");
            this.oglr.drawLine(sg2d, x1, y1, x2, y2);
        }
        
        @Override
        public void drawRect(SunGraphics2D sg2d, int x, int y, int width, int height) {
            GraphicsPrimitive.tracePrimitive("OGLDrawRect");
            this.oglr.drawRect(sg2d, x, y, width, height);
        }
        
        @Override
        protected void drawPoly(SunGraphics2D sunGraphics2D, int[] array, int[] array2, int n, boolean b) {
            GraphicsPrimitive.tracePrimitive("OGLDrawPoly");
            OGLRenderer.access$100(this.oglr, sunGraphics2D, array, array2, n, b);
        }
        
        @Override
        public void fillRect(SunGraphics2D sg2d, int x, int y, int width, int height) {
            GraphicsPrimitive.tracePrimitive("OGLFillRect");
            this.oglr.fillRect(sg2d, x, y, width, height);
        }
        
        @Override
        protected void drawPath(SunGraphics2D sunGraphics2D, Path2D.Float float1, int n, int n2) {
            GraphicsPrimitive.tracePrimitive("OGLDrawPath");
            OGLRenderer.access$200(this.oglr, sunGraphics2D, float1, n, n2);
        }
        
        @Override
        protected void fillPath(SunGraphics2D sunGraphics2D, Path2D.Float float1, int n, int n2) {
            GraphicsPrimitive.tracePrimitive("OGLFillPath");
            OGLRenderer.access$300(this.oglr, sunGraphics2D, float1, n, n2);
        }
        
        @Override
        protected void fillSpans(SunGraphics2D sunGraphics2D, SpanIterator spanIterator, int n, int n2) {
            GraphicsPrimitive.tracePrimitive("OGLFillSpans");
            OGLRenderer.access$400(this.oglr, sunGraphics2D, spanIterator, n, n2);
        }
        
        @Override
        public void fillParallelogram(SunGraphics2D sg2d, double ux1, double uy1, double ux2, double uy2, double x, double y, double dx1, double dy1, double dx2, double dy2) {
            GraphicsPrimitive.tracePrimitive("OGLFillParallelogram");
            this.oglr.fillParallelogram(sg2d, ux1, uy1, ux2, uy2, x, y, dx1, dy1, dx2, dy2);
        }
        
        @Override
        public void drawParallelogram(SunGraphics2D sg2d, double ux1, double uy1, double ux2, double uy2, double x, double y, double dx1, double dy1, double dx2, double dy2, double lw1, double lw2) {
            GraphicsPrimitive.tracePrimitive("OGLDrawParallelogram");
            this.oglr.drawParallelogram(sg2d, ux1, uy1, ux2, uy2, x, y, dx1, dy1, dx2, dy2, lw1, lw2);
        }
        
        public void copyArea(SunGraphics2D sunGraphics2D, int n, int n2, int n3, int n4, int n5, int n6) {
            GraphicsPrimitive.tracePrimitive("OGLCopyArea");
            this.oglr.copyArea(sunGraphics2D, n, n2, n3, n4, n5, n6);
        }
    }
}
